@@ -1,14 +1,14 @@
-from typing import Dict, List, Union, Any
+from typing import Dict, List, Union, Any, Self, TypeVar
 import numpy as np
 import pandas as pd
 
-
 class Collection:
     VType = None
+
     uid = "uid"
-    def __init__(self, data: Union[Dict[str,Any], List[Any]]=None, check_types=True):
+    def __init__(self, data: Union[Dict, List]=None, check_types=True):
         
-        self.data = {}
+        self.data: dict[str] = {}
         if isinstance(data, dict):
             self.data = data
         elif isinstance(data, self.__class__):
@@ -25,6 +25,7 @@ class Collection:
     def __getattr__(self, name):
         if name in self.data:
             return self.data[name]
+        raise AttributeError(f"{name} not found in {self.__class__}")
 
     def __getitem__(self, key: Union[int, str]):
         if isinstance(key, int): 
@@ -44,18 +45,18 @@ class Collection:
     def to_list(self):
         return list(self.data.values())
     
-    def to_dicts(self) -> List[dict]:
+    def to_dicts(self) -> list[dict]:
         return [v.to_dict() for v in self.data.values()]
 
     def to_dict(self) -> Dict[str, dict]:
         return {k: v.to_dict() for k, v in self.data.items()}
 
     @classmethod
-    def from_dicts(cls, vals: List[dict]):
+    def from_dicts(cls, vals: List[dict]) -> Self:
         return cls([cls.VType.from_dict(**v) for v in vals])    
 
     @classmethod
-    def from_dict(cls, vals: Dict[str, dict]):
+    def from_dict(cls, vals: Dict[str, dict]) -> Self:
         return cls([cls.VType.from_dict(v) for v in vals.values()])
     
     def add(self, v):
@@ -65,7 +66,7 @@ class Collection:
             self.data = dict(**self.data, **v.data)
         return v
     
-    def concat(self, vs: list):
+    def concat(self, vs: list[Self]) -> Self:
         coll = self.__class__([])
         for v in vs:
             coll.add(v)
@@ -85,15 +86,15 @@ class Collection:
         else:
             return f"{prefix}{i}"
 
-    def copy(self, deep=True):
+    def copy(self, deep=True) -> Self:
         return self.__class__([v.copy() for v in self] if deep else self.data.copy())
     
     def __str__(self) -> str:
         return str(pd.Series({k: str(v) for k, v in self.data.items()}))
     
     def __repr__(self) -> str:
-        contents = str(pd.Series({k: repr(v) for k, v in self.data.items()}))
-        return f"{self.__class__.__name__}\n{contents}"
+        
+        return str(pd.Series({k: repr(v) for k, v in self.data.items()}))
     
     def __len__(self) -> int:
         return len(self.data)
\ No newline at end of file
