@@ -10,6 +10,7 @@ the higher level parameters and another set of functions to collect the paramete
 elements collection.
 
 """
+from __future__ import annotations
 import enum
 from typing import List, Dict, Callable, Union, Tuple
 import numpy as np
@@ -47,7 +48,7 @@ class ManDef:
         )
 
     @staticmethod
-    def from_dict(data: dict):
+    def from_dict(data: dict) -> ManDef:
         info = ManInfo.from_dict(data["info"])
         mps = ManParms.from_dict(data["mps"])
         eds = ElDefs.from_dict(data["eds"], mps)
@@ -90,7 +91,7 @@ class ManDef:
         
         return ElDef.build(
             Line,
-            f"entry_{self.info.short_name}", 
+            f"entry_line", 
             30.0, 
             max(man_start_x - itrans.translation.x[0] * heading, 30), 
             0)
