@@ -11,6 +11,8 @@ from numbers import Number
 from . import Collector, Collectors, MathOpp, FunOpp, ItemOpp, Opp
 from dataclasses import dataclass, field
 from typing import Any
+import numpy as np
+
 
 @dataclass
 class ManParm(Opp):
@@ -46,7 +48,7 @@ class ManParm(Opp):
         return ManParm(
             name = data["name"],
             criteria = Criteria.from_dict(data["criteria"]),
-            default = data["default"],
+            default = data["default"] if 'default' in data else data['defaul'], # because default is reserverd in javascript
             collectors = Collectors.from_dict(data["collectors"])
         )
 
@@ -63,21 +65,22 @@ class ManParm(Opp):
         self.collectors.data[id] = collector
 
     def collect(self, els):
-        return np.array([collector(els) for collector in self.collectors])
+        return {str(collector): collector(els) for collector in self.collectors}
 
-    def get_downgrades(self, els):
-        values = self.collect(els)
-        return Result(
-            self.name, 
-            values,
-            self.criteria(values),
-            self.collectors.keys()
+    def collect_vis(self, els, state: State):
+        return [np.mean([c.visibility(els, state) for c in colls.list_parms()]) for colls in self.collectors]
+
+    def get_downgrades(self, els, state: State=None):
+        coll = self.collect(els)
+        vs = list(coll.values())
+        meas = Measurement(
+            vs,
+            np.full(len(vs), self.default),
+            np.full(len(vs), 0.4) if state is None else np.array(self.collect_vis(els, state))
         )
 
-#        downgrades = {c.elname: v for c, v in zip(self.collectors, result.downgrades)}
-#        errors = {c.elname: v for c, v in zip(self.collectors, result.errors)}
-#        return [downgrades[e.name] if e.name in downgrades else 0.0 for e in els]
-
+        keys, errors, dgs = self.criteria(list(coll.keys()), list(coll.values())) 
+        return Result(self.name, meas, vs, errors, dgs * meas.visibility, keys)
 
     @property
     def value(self):
@@ -108,12 +111,17 @@ class ManParm(Opp):
     def copy(self):
         return ManParm(name=self.name, criteria=self.criteria, default=self.default, collectors=self.collectors.copy())
 
+    def list_parms(self):
+        return [self]
+
+    def __repr__(self):
+        return f'ManParm({self.name}, {self.criteria.__class__.__name__}, {self.default})'
 
 class ManParms(Collection):
     VType=ManParm
     uid="name"
 
-    def collect(self, manoeuvre: Manoeuvre) -> Results:
+    def collect(self, manoeuvre: Manoeuvre, state: State=None) -> Results:
         """Collect the comparison downgrades for each manparm for a given manoeuvre.
 
         Args:
@@ -122,7 +130,7 @@ class ManParms(Collection):
         Returns:
             Dict[str, float]: The sum of downgrades for each ManParm
         """
-        return Results("Inter",[mp.get_downgrades(manoeuvre.all_elements()) for mp in self if not isinstance(mp.criteria, Combination)])
+        return Results("Inter",[mp.get_downgrades(manoeuvre.all_elements(), state) for mp in self if not isinstance(mp.criteria, Combination)])
     
     def append_collectors(self, colls: Dict[str, Callable]):
         """Append each of a dict of collector methods to the relevant ManParm
@@ -133,7 +141,6 @@ class ManParms(Collection):
         for mp, col in colls.items():
             self.data[mp].append(col)
 
-
     def update_defaults(self, intended: Manoeuvre):
         """Pull the parameters from a manoeuvre object and update the defaults of self based on the result of 
         the collectors.
@@ -143,7 +150,7 @@ class ManParms(Collection):
         """
 
         for mp in self:
-            flown_parm = mp.collect(intended.all_elements())
+            flown_parm = list(mp.collect(intended.all_elements()).values())
             if len(flown_parm) > 0:
                 if mp.default is not None:
                     if isinstance(mp.criteria, Combination):
