@@ -13,12 +13,13 @@ from .funopp import FunOpp
 
 @dataclass
 class ItemOpp(Opp):
-    """This class creates an Operation that returns a single item from a combination ManParm"""
+    """This class creates an Operation that returns a single item,
+        usually from a Combination manparm"""
     a: Any
     item: int
     
-    def __call__(self, mps):
-        return self.a.valuefunc(self.item)(mps)
+    def __call__(self, mps, **kwargs):
+        return self.a.valuefunc(self.item)(mps, **kwargs)
     
     def __str__(self):
         return f"{self.a.name}[{self.item}]"
@@ -48,3 +49,8 @@ class ItemOpp(Opp):
     def __abs__(self):
         return FunOpp(self.name, self, "abs")
 
+    def list_parms(self):
+        if isinstance(self.a, Opp):
+            return self.a.list_parms()
+        else:
+            return []
\ No newline at end of file
