@@ -4,7 +4,8 @@ from geometry import Transformation, PX, PY, PZ, P0
 from flightanalysis.state import State
 from flightanalysis.base import Time
 from .element import Element
-from flightanalysis.schedule.scoring import *
+from flightanalysis.schedule.scoring.criteria.f3a_criteria import F3A
+from flightanalysis.schedule.scoring import Measurement, DownGrade, DownGrades
 
 
 class StallTurn(Element):
@@ -16,17 +17,10 @@ class StallTurn(Element):
     @property
     def intra_scoring(self) -> DownGrades:
         return DownGrades([
-            DownGrade(Measurement.roll_angle, f3a.intra_roll)
+            DownGrade(Measurement.roll_angle, F3A.intra.roll),
+            #DownGrade(Measurement.track_y, F3A.single.track)  # track_y doesn't work as template velocity is zero
         ])
 
-    def to_dict(self):
-        return dict(
-            kind=self.__class__.__name__,
-            yaw_rate=self.yaw_rate,
-            speed=self.speed,
-            uid=self.uid
-        )
-
     def describe(self):
         return f"stallturn, yaw rate = {self.yaw_rate}"
 
@@ -52,4 +46,7 @@ class StallTurn(Element):
     def copy_direction(self, other) -> StallTurn:
         return self.set_parms(
             yaw_rate=abs(self.yaw_rate) * np.sign(other.yaw_rate)
-        )
\ No newline at end of file
+        )
+    
+    def yaw_rate_visibility(self, st: State):
+        return [0.4]
\ No newline at end of file
