@@ -28,7 +28,7 @@ class Manoeuvre:
 
     def to_dict(self):
         return dict(
-            entry_line=self.entry_line.to_dict() if self.entry_line else None,
+            entry_line=self.entry_line.to_dict(exit_only=True) if self.entry_line else None,
             elements=self.elements.to_dicts(),
             exit_line=self.exit_line.to_dict() if self.exit_line else None,
             uid=self.uid
@@ -47,7 +47,7 @@ class Manoeuvre:
         )
 
     def all_elements(self, create_entry: bool = False, create_exit: bool = False) -> Elements:
-        els = Elements()
+        els = Elements([])
 
         if self.entry_line:
             els.add(self.entry_line)
@@ -104,10 +104,7 @@ class Manoeuvre:
         els = self.all_elements()
         for i, elm in enumerate(els):
             st = elm.get_data(aligned)
-            elms.add(elm.match_intention(
-                templates[-1][-1].transform, 
-                st
-            ))
+            elms.add(elm.match_intention(templates[-1][-1].transform, st))
 
             if isinstance(elms[-1], Autorotation):
                 #copy the autorotation pitch offset back to the preceding pitch departure
@@ -117,16 +114,31 @@ class Manoeuvre:
                 elms[-2].break_angle = pos_break if pos_break > -neg_break else neg_break
                 
             templates.append(elms[-1].create_template(
-                templates[-1][-1], 
+                templates[-1][-1].relocate(st[0].pos), 
                 st.time.extend() if i < len(els) - 1 else st.time
             ))
                     
         return Manoeuvre.from_all_elements(self.uid, elms), State.stack(templates[1:]).label(manoeuvre=self.uid)
 
+    def el_matched_tp(self, istate: State, aligned: State) -> State:
+        els = self.all_elements()
+        aligned= self.get_data(aligned)
+        templates = [istate]
+        for i, el in enumerate(els):
+            st = el.get_data(aligned)
+            templates.append(el.create_template(
+                templates[-1][-1].relocate(st[0].pos), 
+                st.time.extend() if i < len(els) - 1 else st.time
+            ))
+        return State.stack(templates[1:])
+
+    def copy(self):
+        return Manoeuvre.from_all_elements(self.uid, self.all_elements().copy(deep=True))
+
     def copy_directions(self, other: Manoeuvre) -> Manoeuvre:
         return Manoeuvre.from_all_elements(
             self.uid, 
-            [es.copy_direction(eo) for es, eo in zip(self.all_elements(), other.all_elements())]
+            Elements(self.all_elements().copy_directions(other.all_elements()))
         )
 
     def analyse(self, flown: State, template: State):
@@ -142,4 +154,7 @@ class Manoeuvre:
         return ElementsResults(ers)
 
     def descriptions(self):
-        return [e.describe() for e in self.elements]
\ No newline at end of file
+        return [e.describe() for e in self.elements]
+    
+    def __repr__(self):
+        return f'Manoeuvre({self.uid}, len={len(self.elements)})'
\ No newline at end of file
