@@ -1,32 +1,30 @@
 from __future__ import annotations
-from typing import Union, List
+from typing import Union, List, Tuple, Self
 import warnings
 from pathlib import Path
-
 import numpy as np
 import pandas as pd
 from pandas.api.types import is_list_like
-
-
 from geometry import Point, Quaternion, Transformation, PX, PY, PZ, P0, Q0, Coord
-
 from flightanalysis import Table, Constructs, SVar, Time, FlightLine, Box, Flow, Environment
 
 from warnings import warn
 try:
     from scipy.spatial.distance import euclidean
+except ImportError as ex:
+    pass
+try:
     from fastdtw import fastdtw
 except ImportError as ex:
     pass
 
 
-
 class State(Table):
     constructs = Table.constructs + Constructs([
         SVar("pos", Point,       ["x", "y", "z"]           , lambda self: P0(len(self))       ), 
         SVar("att", Quaternion,  ["rw", "rx", "ry", "rz"]  , lambda self : Q0(len(self))       ),
         SVar("vel", Point,       ["u", "v", "w"]           , lambda st: P0() if len(st)==1 else st.att.inverse().transform_point(st.pos.diff(st.dt))  ),
-        SVar("rvel", Point,       ["p", "q", "r"]           , lambda st: P0() if len(st)==1 else st.att.body_diff(st.dt).remove_outliers(3)  ),
+        SVar("rvel", Point,       ["p", "q", "r"]           , lambda st: P0() if len(st)==1 else st.att.body_diff(st.dt)),
         SVar("acc", Point,       ["du", "dv", "dw"]        , lambda st : P0() if len(st)==1 else st.att.inverse().transform_point(st.att.transform_point(st.vel).diff(st.dt) + PZ(9.81, len(st)))),
         SVar("racc", Point,       ["dp", "dq", "dr"]        , lambda st: P0() if len(st)==1 else st.rvel.diff(st.dt)),
     ])
@@ -39,9 +37,11 @@ class State(Table):
     @property
     def back_transform(self):
         return Transformation(-self.pos, self.att.inverse())
-     
+
     @staticmethod
-    def from_transform(transform: Transformation, **kwargs):
+    def from_transform(transform: Transformation=None, **kwargs):
+        if transform is None:
+            transform = Transformation()
         if not "time" in kwargs: 
             kwargs["time"] = Time.from_t(np.linspace(0, State._construct_freq*len(transform), len(transform)))
         return State.from_constructs(pos=transform.p, att=transform.q, **kwargs)
@@ -75,21 +75,14 @@ class State(Table):
         pos = (att.transform_point(vel) * time.dt).cumsum() + self.pos
         return State.from_constructs(time,pos, att, vel, rvel)
 
-    def extrapolate(self: State, duration: float) -> State:
-        """extrapolate the input state, currently ignores input accelerations
-
-        Args:
-            istate (State): initial state of length 1
-            duration (float): duration of extrapolation in seconds
+    def extrapolate(self, duration: float, min_len=3) -> State:
+        """Extrapolate the input state, currently ignores input accelerations
 
         Returns:
             State: state projected forwards
         """
-
-        npoints = np.max([int(np.ceil(duration / self.dt[0])), 3])
-
+        npoints = np.max([int(np.ceil(duration / self.dt[0])), min_len])
         time = Time.from_t(np.linspace(0,duration, npoints))
-
         return self.fill(time)
 
     @staticmethod
@@ -104,7 +97,7 @@ class State(Table):
         return State(df.set_index("t", drop=False))
 
     @staticmethod
-    def from_flight(flight, box:Union[FlightLine, Box, str]) -> State:
+    def from_flight(flight, box:Union[FlightLine, Box, str] = None) -> State:
         from flightdata import Flight, Fields
         if isinstance(flight, str):
             flight = {
@@ -155,19 +148,13 @@ class State(Table):
         return State.from_constructs(time, pos, att, vel, rvel, acc)
 
     @staticmethod
-    def stack(sections: list) -> State:
-        """stack a list of States on top of each other. last row of each is replaced with first row of the next, 
-            indexes are offset so they are sequential
-
-        Args:
-            States (List[State]): list of States to stacked
-
-        Returns:
-            State: the resulting State
+    def stack(sections: list) -> Self:
+        """Stack a list of States on top of each other. last row of each is replaced with first row of the next, 
+            indexes are offset so they are sequential. 
+            TODO this should move into the parent class.
         """
         # first build list of index offsets, to be added to each dataframe
-        offsets = [0] + [sec.duration for sec in sections[:-1]]
-        offsets = np.cumsum(offsets)
+        offsets = np.cumsum([0] + [sec.duration for sec in sections[:-1]])
 
         # The sections to be stacked need their last row removed, as the first row of the next replaces it
         dfs = [section.data.iloc[:-1] for section in sections[:-1]] + \
@@ -183,37 +170,25 @@ class State(Table):
 
         return State(combo)
 
-
-
     @staticmethod
-    def align(flown: State, template: State, radius=5, mirror=True, white=False, weights = Point(1,1,1)) -> State:
+    def align(
+        flown: State, 
+        template: State, 
+        radius=5, mirror=True,
+        weights = Point(1,1.2,0.5),
+        tp_weights = Point(0.6,0.6,0.6),
+    ) -> Tuple(float, Self):
         """Perform a temporal alignment between two sections. return the flown section with labels 
-        copied from the template along the warped path
-
-        Args:
-            flown (Section): An un-labelled Section
-            template (Section): A labelled Section
-            radius (int, optional): The DTW search radius. Defaults to 5.
-            whiten (bool, optional): Whether to whiten the data before performing the alignment. Defaults to False.
-
+        copied from the template along the warped path. 
         """
-        
-        if white:
-            warnings.filterwarnings("ignore", message="Some columns have standard deviation zero. ")
-
         def get_brv(brv):
             if mirror:
-                brv.data[:,0] = abs(brv.data[:,0])
-                brv.data[:,2] = abs(brv.data[:,2])
-
-            if white:
-                brv = brv.whiten()
-
+                brv = brv.abs() * Point(1, 0, 1) + brv * Point(0, 1, 0 )
             return brv * weights
 
         fl = get_brv(flown.rvel)
 
-        tp = get_brv(template.rvel)
+        tp = get_brv(template.rvel * tp_weights)
 
         distance, path = fastdtw(
             tp.data,
@@ -222,72 +197,47 @@ class State(Table):
             dist=euclidean
         )
 
-        return distance, State.copy_labels(template, flown, path)
+        return distance, State.copy_labels(template, flown, path, 2)
 
-    @staticmethod
-    def copy_labels(template: State, flown: State, path=None) -> State:
-        """Copy the labels from a template section to a flown section along the index warping path
-
-        Args:
-            template (Section): A labelled section
-            flown (Section): An unlabelled section
-            path (List): A list of lists containing index pairs from template to flown
-
-        Returns:
-            Section: a labelled section
-        """
-
-        flown = flown.remove_labels()
-
-        keys = [k for k in ["manoeuvre", "element", "sub_element"] if k in template.data.columns]
-        if path is None:
-            return State(
-                pd.concat(
-                    [flown.data.reset_index(drop=True), template.data.loc[:,keys].reset_index(drop=True)], 
-                    axis=1
-                ).set_index("t", drop=False)
-            )
-        else:
-            mans = pd.DataFrame(path, columns=["template", "flight"]).set_index("template").join(
-                    template.data.reset_index(drop=True).loc[:, keys]
-                ).groupby(['flight']).last().reset_index().set_index("flight")
-
-            return State(flown.data.reset_index(drop=True).join(mans).set_index("t", drop=False))
-
-
-    def splitter_labels(self: State, mans: List[dict]) -> State:
+    def splitter_labels(self: State, mans: List[dict], better_names: List[str]=None) -> State:
             """label the manoeuvres in a State based on the flight coach splitter information
 
             TODO this assumes the state only contains the dataset contained in the json
 
             Args:
                 mans (list): the mans field of a flight coach json
+                better_names: names to replace the splitter names with. does not include takeoff or landing.
 
             Returns:
                 State: State with labelled manoeuvres
             """
 
+                
             takeoff = self.data.iloc[0:int(mans[0]["stop"])+1]
 
             labels = [mans[0]["name"]]
             labelled = [State(takeoff).label(manoeuvre=labels[0])]
-            
-            for split_man in mans[1:]:
+            if better_names:
+                better_names.append('land')
+
+            for i, split_man in enumerate(mans[1:]):
                 
                 while split_man["name"] in labels:
                     split_man["name"] = split_man["name"] + "2"
 
+
+                name = better_names[i] if better_names else split_man["name"]
+
                 labelled.append(
                     State(
                         self.data.iloc[int(split_man["start"]):int(split_man["stop"])+1]
-                    ).label(manoeuvre=split_man["name"])
+                    ).label(manoeuvre=name)
                 )
                 labels.append(split_man["name"])
 
             return State.stack(labelled)
 
-
-    def get_subset(self: State, mans: Union[list, slice], col="manoeuvre"):
+    def get_subset(self: State, mans: Union[list, slice], col="manoeuvre", min_len=1) -> Self:
         selectors = self.data.loc[:,col].unique()
         if isinstance(mans, slice):
             mans = selectors[mans]
@@ -302,14 +252,20 @@ class State(Table):
             
         assert all(isinstance(m, str) for m in mans)
 
-        return State(self.data.loc[self.data.loc[:, col].isin(mans)])
+        return State(self.data.loc[self.data.loc[:, col].isin(mans)], False, min_len)
 
-    def get_manoeuvre(self: State, manoeuvre: Union[str, list, int]):
+    def get_manoeuvre(self: State, manoeuvre: Union[str, list, int]) -> Self:
         return self.get_subset(manoeuvre, "manoeuvre")
 
-    def get_element(self: State, element: Union[str, list, int]):
+    def get_element(self: State, element: Union[str, list, int]) -> Self:
         return self.get_subset(element, "element") 
 
+    def get_man_or_el(self: State, el: str) -> Self:
+        if el in self.data.element.unique():
+            return self.get_element([el])
+        elif el in self.data.manoeuvre.unique():
+            return self.get_manoeuvre([el])
+        
     def get_meid(self: State, manid: int, elid: int=None):
         st = self.get_manoeuvre(manid)
         if not elid is None:
@@ -318,6 +274,7 @@ class State(Table):
             return st
 
     def convert_state(self: State, r: Point) -> State:
+        """Rotate body axis by an axis angle"""
         att = self.att.body_rotate(r)
         q =  att.inverse() * self.att
         return State.from_constructs(
@@ -391,29 +348,35 @@ class State(Table):
         return body_axis
 
     def _create_json_data(self: State) -> pd.DataFrame:
-        fcd = pd.DataFrame(columns=["N", "E", "D", "VN", "VE", "VD", "r", "p", "yw", "wN", "wE", "roll", "pitch", "yaw"])
-        fcd["N"], fcd["E"], fcd["D"] = self.x, -self.y, -self.z
-        wvels = self.body_to_world(Point(self.vel))
-        fcd["VN"], fcd["VE"], fcd["VD"] = wvels.x, -wvels.y, -wvels.z
+
+        wvels = self.transform.rotate(self.vel)
 
         transform = Transformation.from_coords(
             Coord.from_xy(Point(0, 0, 0), Point(1, 0, 0), Point(0, 1, 0)),
             Coord.from_xy(Point(0, 0, 0), Point(1, 0, 0), Point(0, -1, 0))
         )
-        eul = transform.rotate(Quaternion(self.att)).to_euler()
-        ex, ey, ez = eul.x, eul.y, eul.z
-        fcd["roll"], fcd["pitch"], fcd["yaw"] = ex, ey, ez
+        eul = transform.rotate(self.att).to_euler()
 
-        fcd["r"] = np.degrees(fcd["roll"])
-        fcd["p"] = np.degrees(fcd["pitch"])
-        fcd["yw"] = np.degrees(fcd["yaw"])
+        fcd = pd.DataFrame(
+            data = dict(
+                time = self.t * 1e6,
+                N=self.x, 
+                E=-self.y, 
+                D=-self.z, 
+                VN=wvels.x, 
+                VE=-wvels.y, 
+                VD=-wvels.z, 
+                r=np.degrees(eul.x), 
+                p=np.degrees(eul.y), 
+                yw=np.degrees(eul.z), 
+                wN=np.zeros(len(self)), 
+                wE=np.zeros(len(self)), 
+                roll=eul.x, 
+                pitch=eul.y, 
+                yaw=eul.z
+            ),
+        )
 
-        fcd["wN"] = np.zeros(len(ex))
-        fcd["wE"] = np.zeros(len(ex))
-
-        fcd = fcd.reset_index()
-        fcd.columns = ["time", "N", "E", "D", "VN", "VE", "VD", "r", "p", "yw", "wN", "wE", "roll", "pitch", "yaw"]
-        fcd["time"] = np.int64(fcd["time"] * 1e6)
         return fcd
 
     def _create_json_mans(self: State, sdef) -> pd.DataFrame:
@@ -476,7 +439,7 @@ class State(Table):
                 "pilotLng": "0.0",
                 "pilotAlt": "0.00",
                 "centerLat": "0.0",
-                "centerLng": "0.0",
+                "centerLng": "-0.1",
                 "centerAlt": "0.00",
                 "schedule": [schedule_category, schedule_name]
             },
@@ -527,14 +490,13 @@ class State(Table):
         self = self.move(Transformation(-transform.pos, Q0()))
         return self.move(Transformation(P0(), transform.att.inverse()))
 
-
     def relocate(self:State, start_pos: Point) -> State:
         offset = start_pos - self.pos[0]
         return self.move(Transformation(offset, Q0()))
 
     def superimpose_angles(self: State, angles: Point, reference:str="body"): 
         assert reference in ["body", "world"]
-        sec = State.copy_labels(
+        return State.copy_labels(
             self, 
             State.from_constructs(
                 self.time,
@@ -543,11 +505,6 @@ class State(Table):
             )
         ) 
 
-        #if "sub_element" in self.data.columns:
-        #   sec = sec.append_columns(self.data["sub_element"])
-        return sec
-
-
     def superimpose_rotation(self: State, axis: Point, angle: float, reference:str="body"):
         """Generate a new section, identical to self, but with a continous rotation integrated
         """
@@ -560,7 +517,6 @@ class State(Table):
         return self.superimpose_angles(angles, reference)
 
 
-
     def superimpose_roll(self: State, angle: float) -> State:
         """Generate a new section, identical to self, but with a continous roll integrated
 
@@ -569,7 +525,6 @@ class State(Table):
         """
         return self.superimpose_rotation(PX(), angle)
 
-
     def smooth_rotation(self: State, axis: Point, angle: float, reference:str="body", w: float=0.25, w2=0.1):
         """Accelerate for acc_prop * t, flat rate for middle, slow down for acc_prop * t.
 
@@ -601,3 +556,11 @@ class State(Table):
         angles = Point.full(axis.unit(), len(t)) * np.concatenate([angles_0, angles_1, angles_2])
 
         return self.superimpose_angles(angles, reference)
+    
+    def zero_g_acc(self):
+        return self.att.inverse().transform_point(PZ(-9.81)) + self.acc
+
+    def arc_centre(self) -> Point:
+        acc = Point.vector_rejection(self.zero_g_acc(), self.vel)
+
+        return acc.unit() * abs(self.vel) ** 2 / abs(acc)
\ No newline at end of file
